// Telegrammer - Telegram Bot Swift SDK.
// This file is autogenerated by API/generate_wrappers.rb script.

/**
 This object represents a message.

 SeeAlso Telegram Bot API Reference:
 [Message](https://core.telegram.org/bots/api#message)
 */
public final class Message: Codable {

    /// Custom keys for coding/decoding `Message` struct
    enum CodingKeys: String, CodingKey {
        case messageId = "message_id"
        case from = "from"
        case date = "date"
        case chat = "chat"
        case forwardFrom = "forward_from"
        case forwardFromChat = "forward_from_chat"
        case forwardFromMessageId = "forward_from_message_id"
        case forwardSignature = "forward_signature"
        case forwardDate = "forward_date"
        case replyToMessage = "reply_to_message"
        case editDate = "edit_date"
        case mediaGroupId = "media_group_id"
        case authorSignature = "author_signature"
        case text = "text"
        case entities = "entities"
        case captionEntities = "caption_entities"
        case audio = "audio"
        case document = "document"
        case animation = "animation"
        case game = "game"
        case photo = "photo"
        case sticker = "sticker"
        case video = "video"
        case voice = "voice"
        case videoNote = "video_note"
        case caption = "caption"
        case contact = "contact"
        case location = "location"
        case venue = "venue"
        case newChatMembers = "new_chat_members"
        case leftChatMember = "left_chat_member"
        case newChatTitle = "new_chat_title"
        case newChatPhoto = "new_chat_photo"
        case deleteChatPhoto = "delete_chat_photo"
        case groupChatCreated = "group_chat_created"
        case supergroupChatCreated = "supergroup_chat_created"
        case channelChatCreated = "channel_chat_created"
        case migrateToChatId = "migrate_to_chat_id"
        case migrateFromChatId = "migrate_from_chat_id"
        case pinnedMessage = "pinned_message"
        case invoice = "invoice"
        case successfulPayment = "successful_payment"
        case connectedWebsite = "connected_website"
        case passportData = "passport_data"
    }

    /// Unique message identifier inside this chat
    public var messageId: Int

    /// Optional. Sender, empty for messages sent to channels
    public var from: User?

    /// Date the message was sent in Unix time
    public var date: Int

    /// Conversation the message belongs to
    public var chat: Chat

    /// Optional. For forwarded messages, sender of the original message
    public var forwardFrom: User?

    /// Optional. For messages forwarded from channels, information about the original channel
    public var forwardFromChat: Chat?

    /// Optional. For messages forwarded from channels, identifier of the original message in the channel
    public var forwardFromMessageId: Int?

    /// Optional. For messages forwarded from channels, signature of the post author if present
    public var forwardSignature: String?

    /// Optional. For forwarded messages, date the original message was sent in Unix time
    public var forwardDate: Int?

    /// Optional. For replies, the original message. Note that the Message object in this field will not contain further reply_to_message fields even if it itself is a reply.
    public var replyToMessage: Message?

    /// Optional. Date the message was last edited in Unix time
    public var editDate: Int?

    /// Optional. The unique identifier of a media message group this message belongs to
    public var mediaGroupId: String?

    /// Optional. Signature of the post author for messages in channels
    public var authorSignature: String?

    /// Optional. For text messages, the actual UTF-8 text of the message, 0-4096 characters.
    public var text: String?

    /// Optional. For text messages, special entities like usernames, URLs, bot commands, etc. that appear in the text
    public var entities: [MessageEntity]?

    /// Optional. For messages with a caption, special entities like usernames, URLs, bot commands, etc. that appear in the caption
    public var captionEntities: [MessageEntity]?

    /// Optional. Message is an audio file, information about the file
    public var audio: Audio?

    /// Optional. Message is a general file, information about the file
    public var document: Document?

    /// Optional. Message is an animation, information about the animation. For backward compatibility, when this field is set, the document field will also be set
    public var animation: Animation?

    /// Optional. Message is a game, information about the game. More about games »
    public var game: Game?

    /// Optional. Message is a photo, available sizes of the photo
    public var photo: [PhotoSize]?

    /// Optional. Message is a sticker, information about the sticker
    public var sticker: Sticker?

    /// Optional. Message is a video, information about the video
    public var video: Video?

    /// Optional. Message is a voice message, information about the file
    public var voice: Voice?

    /// Optional. Message is a video note, information about the video message
    public var videoNote: VideoNote?

    /// Optional. Caption for the animation, audio, document, photo, video or voice, 0-1024 characters
    public var caption: String?

    /// Optional. Message is a shared contact, information about the contact
    public var contact: Contact?

    /// Optional. Message is a shared location, information about the location
    public var location: Location?

    /// Optional. Message is a venue, information about the venue
    public var venue: Venue?

    /// Optional. New members that were added to the group or supergroup and information about them (the bot itself may be one of these members)
    public var newChatMembers: [User]?

    /// Optional. A member was removed from the group, information about them (this member may be the bot itself)
    public var leftChatMember: User?

    /// Optional. A chat title was changed to this value
    public var newChatTitle: String?

    /// Optional. A chat photo was change to this value
    public var newChatPhoto: [PhotoSize]?

    /// Optional. Service message: the chat photo was deleted
    public var deleteChatPhoto: Bool?

    /// Optional. Service message: the group has been created
    public var groupChatCreated: Bool?

    /// Optional. Service message: the supergroup has been created. This field can‘t be received in a message coming through updates, because bot can’t be a member of a supergroup when it is created. It can only be found in reply_to_message if someone replies to a very first message in a directly created supergroup.
    public var supergroupChatCreated: Bool?

    /// Optional. Service message: the channel has been created. This field can‘t be received in a message coming through updates, because bot can’t be a member of a channel when it is created. It can only be found in reply_to_message if someone replies to a very first message in a channel.
    public var channelChatCreated: Bool?

    /// Optional. The group has been migrated to a supergroup with the specified identifier. This number may be greater than 32 bits and some programming languages may have difficulty/silent defects in interpreting it. But it is smaller than 52 bits, so a signed 64 bit integer or double-precision float type are safe for storing this identifier.
    public var migrateToChatId: Int64?

    /// Optional. The supergroup has been migrated from a group with the specified identifier. This number may be greater than 32 bits and some programming languages may have difficulty/silent defects in interpreting it. But it is smaller than 52 bits, so a signed 64 bit integer or double-precision float type are safe for storing this identifier.
    public var migrateFromChatId: Int64?

    /// Optional. Specified message was pinned. Note that the Message object in this field will not contain further reply_to_message fields even if it is itself a reply.
    public var pinnedMessage: Message?

    /// Optional. Message is an invoice for a payment, information about the invoice. More about payments »
    public var invoice: Invoice?

    /// Optional. Message is a service message about a successful payment, information about the payment. More about payments »
    public var successfulPayment: SuccessfulPayment?

    /// Optional. The domain name of the website on which the user has logged in. More about Telegram Login »
    public var connectedWebsite: String?

    /// Optional. Telegram Passport data
    public var passportData: PassportData?


    public init (messageId: Int, from: User? = nil, date: Int, chat: Chat, forwardFrom: User? = nil, forwardFromChat: Chat? = nil, forwardFromMessageId: Int? = nil, forwardSignature: String? = nil, forwardDate: Int? = nil, replyToMessage: Message? = nil, editDate: Int? = nil, mediaGroupId: String? = nil, authorSignature: String? = nil, text: String? = nil, entities: [MessageEntity]? = nil, captionEntities: [MessageEntity]? = nil, audio: Audio? = nil, document: Document? = nil, animation: Animation? = nil, game: Game? = nil, photo: [PhotoSize]? = nil, sticker: Sticker? = nil, video: Video? = nil, voice: Voice? = nil, videoNote: VideoNote? = nil, caption: String? = nil, contact: Contact? = nil, location: Location? = nil, venue: Venue? = nil, newChatMembers: [User]? = nil, leftChatMember: User? = nil, newChatTitle: String? = nil, newChatPhoto: [PhotoSize]? = nil, deleteChatPhoto: Bool? = nil, groupChatCreated: Bool? = nil, supergroupChatCreated: Bool? = nil, channelChatCreated: Bool? = nil, migrateToChatId: Int64? = nil, migrateFromChatId: Int64? = nil, pinnedMessage: Message? = nil, invoice: Invoice? = nil, successfulPayment: SuccessfulPayment? = nil, connectedWebsite: String? = nil, passportData: PassportData? = nil) {
        self.messageId = messageId
        self.from = from
        self.date = date
        self.chat = chat
        self.forwardFrom = forwardFrom
        self.forwardFromChat = forwardFromChat
        self.forwardFromMessageId = forwardFromMessageId
        self.forwardSignature = forwardSignature
        self.forwardDate = forwardDate
        self.replyToMessage = replyToMessage
        self.editDate = editDate
        self.mediaGroupId = mediaGroupId
        self.authorSignature = authorSignature
        self.text = text
        self.entities = entities
        self.captionEntities = captionEntities
        self.audio = audio
        self.document = document
        self.animation = animation
        self.game = game
        self.photo = photo
        self.sticker = sticker
        self.video = video
        self.voice = voice
        self.videoNote = videoNote
        self.caption = caption
        self.contact = contact
        self.location = location
        self.venue = venue
        self.newChatMembers = newChatMembers
        self.leftChatMember = leftChatMember
        self.newChatTitle = newChatTitle
        self.newChatPhoto = newChatPhoto
        self.deleteChatPhoto = deleteChatPhoto
        self.groupChatCreated = groupChatCreated
        self.supergroupChatCreated = supergroupChatCreated
        self.channelChatCreated = channelChatCreated
        self.migrateToChatId = migrateToChatId
        self.migrateFromChatId = migrateFromChatId
        self.pinnedMessage = pinnedMessage
        self.invoice = invoice
        self.successfulPayment = successfulPayment
        self.connectedWebsite = connectedWebsite
        self.passportData = passportData
    }
}
